// amiraToD.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 1/12/04

#include "stdafx.h"

#pragma once
#include <iostream>

// USAGE  prints usage message to screen
void usage()
{
	printf("Converts an amira hypermesh file to a dyna3d .d file\n");
	printf("Usage: amiraToD [amira mesh file (.hmascii)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -enum <numElems (int)>		 = number of elements in mesh; default '0'\n");
	printf("  -nnum <numNodes (int)>		 = number of nodes in mesh; default '0'\n");
	printf("  -o <outfile (.d)>          = output file name (.d); default 'out.d'\n");
	printf("  -val <materialVal (int)>   = value to use for element material; defalut '1'\n");
    exit(-1);
}
// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *famira,*fout;
	int numNodes=0, numElems=0;
	int materialVal=1;

	famira = fail_or_open(argv[1], "rt");
	FileCloser fc1(famira);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-enum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&numElems)){
				printf("entered invalid number of elements");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-nnum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&numNodes)){
				printf("entered invalid number of nodes");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-val")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&materialVal)){
				printf("entered invalid number for new shell value");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.d", "wt");
		printf("writing to file : out.d\n");
	}
	FileCloser fc3(fout);

	printf("number of elements used: %d\n",numElems);
	printf("number of nodes used: %d\n",numNodes);
	printf("assigning material value: %d\n",materialVal);
	printf("converting to d from file: %s\n\n",argv[1]);


	//print header to d file
	/* cards 1-9 */
	fprintf(fout, "                                                                        88 large\n");
	fprintf(fout,"    1%10d%10d         0         0         0         0 0.000E+00  0.0\n"
		,numNodes,numElems);
	fprintf(fout,"    0    0    0    0    0    0    0    0    1\n");
	fprintf(fout,"         0    0    0    0    0e20.0    0 0.000E+00\n");
	fprintf(fout,"    0    0    0    0    0    0    0    0    0    0    0    0    0    0    0    0\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00    0    0 0.000E+00 0.000E+00    0    0 0.000E+00\n");
	fprintf(fout,"    0    0    0    0    0    0    0    0    0              0    0    0    0\n");
	fprintf(fout,"    0    0 0.000E+00    0    0    0    0  250 1.000E-03 9.950E-01 0.000E+00    0\n");
	fprintf(fout,"    0    0    0    0 0.000E+00    0    0    0\n");
	/* material cards */
	fprintf(fout,"    1    00.0000E+00    0    00.0000E+00    00.0000E+000.0000E+00    0    0    0\n");
	fprintf(fout,"                                                                                \n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");
	fprintf(fout," 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00 0.000E+00\n");

	//find nodes in amira file
	char line[1000];

	if(NULL==fgets(line, sizeof(line), famira) ){
		printf("Failed to read first line of amira file\n");
		return -3;
	}
	while (NULL==strstr(line,"*node")){
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file before finding '*node'\n");
			return -4;
		}
	}

	//print nodes to d file
	int nodeNum=0;
	double nodeCord[3]={0};

	while (NULL!=strstr(line,"*node")){
		sscanf(line,"%*c%*c%*c%*c%*c%*c%d%*c%lf%*c%lf%*c%lf",&nodeNum,&nodeCord[0],&nodeCord[1],&nodeCord[2]);
		fprintf(fout,"%8d    0%20e%20e%20e    0\n",nodeNum,nodeCord[0],nodeCord[1],nodeCord[2]);
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file after finding nodes\n");
			return -5;
		}
	}

	//finding tetra element in amira file
	while (NULL==strstr(line,"*tetra")){
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file before finding '*node'\n");
			return -6;
		}
	}

	//print element nodes to d file
	int elemNum=0;
	int elemVal[4]={0};

	while (NULL!=strstr(line,"*tetra")){
		sscanf(line,"%*c%*c%*c%*c%*c%*c%*c%*c%d%*c%*c%*c%d%*c%d%*c%d%*c%d"
			,&elemNum,&elemVal[0],&elemVal[1],&elemVal[2],&elemVal[3]);
		fprintf(fout,"%8d%5d%8d%8d%8d%8d%8d%8d%8d%8d\n"
			,elemNum,materialVal
			,elemVal[0],elemVal[1],elemVal[2],elemVal[3]
			,elemVal[3],elemVal[3],elemVal[3],elemVal[3]);
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file after finding tetra components\n");
			return -7;
		}
	}

	printf("Finished printing to d file.\n");
	return 0;
}