// amiraToK.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#pragma once
#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
	printf("Converts an amira hypermesh file to a lsdyna .k file\n");
	printf("Usage: amiraToK [amira mesh file (.hmascii)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.k)>          = output file name (.k); default 'out.k'\n");
	printf("  -tri                       = use amira tri elements for creating k elements; default 'uses tetra'\n");
	printf("  -val <materialVal (int)>   = value to use for element material; defalut '1'\n");
    exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *famira,*fout;
	int materialVal=1;
	bool doTri=false;

	famira = fail_or_open(argv[1], "rt");
	FileCloser fc1(famira);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-tri")){
			printf("using amira element type: tri\n");
			doTri=true;
		}else if(0==strcmp(argv[i],"-val")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&materialVal)){
				printf("entered invalid number for new shell value");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	FileCloser fc3(fout);
	if(!doTri){
		printf("using amira element type: tetra\n");
	}

	printf("assigning material value: %d\n",materialVal);
	printf("converting to k from file: %s\n\n",argv[1]);

	//print header
	fprintf(fout,"*KEYWORD\n");

	//find nodes in amira file
	char line[1000];

	if(NULL==fgets(line, sizeof(line), famira) ){
		printf("Failed to read first line of amira file\n");
		return -3;
	}
	while (NULL==strstr(line,"*node")){
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file before finding '*node'\n");
			return -4;
		}
	}

	//print nodes to k file
	int nodeNum=0;
	double nodeCord[3]={0};

	fprintf(fout,"*NODE\n");
	while (NULL!=strstr(line,"*node")){
		sscanf(line,"%*c%*c%*c%*c%*c%*c%d%*c%lf%*c%lf%*c%lf",&nodeNum,&nodeCord[0],&nodeCord[1],&nodeCord[2]);
		fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",nodeNum,nodeCord[0],nodeCord[1],nodeCord[2]);
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file after finding nodes\n");
			return -5;
		}
	}

	//finding element in amira file
	if(doTri){
		while (NULL==strstr(line,"*tri")){
			if(NULL == fgets(line, sizeof(line), famira)){
				printf("Reached end of amira file before finding '*tri'\n");
				return -6;
			}
		}
	}else{
		while (NULL==strstr(line,"*tetra")){
			if(NULL == fgets(line, sizeof(line), famira)){
				printf("Reached end of amira file before finding '*tetra'\n");
				return -6;
			}
		}
	}

	//print element nodes to k file
	nodeNum=0;
	int nodeVal[4]={0};

	fprintf(fout,"*ELEMENT_SOLID\n");
	while (NULL!=strstr(line,"*t")){
		if(doTri){
			sscanf(line,"%*c%*c%*c%*c%*c%*c%d%*c%*c%*c%d%*c%d%*c%d"
			,&nodeNum,&nodeVal[0],&nodeVal[1],&nodeVal[2]);
			nodeVal[3]=nodeVal[2];
		}else{
            sscanf(line,"%*c%*c%*c%*c%*c%*c%*c%*c%d%*c%*c%*c%d%*c%d%*c%d%*c%d"
			,&nodeNum,&nodeVal[0],&nodeVal[1],&nodeVal[2],&nodeVal[3]);
		}
		fprintf(fout,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n"
			,nodeNum,materialVal
			,nodeVal[0],nodeVal[1],nodeVal[2],nodeVal[3]
			,nodeVal[3],nodeVal[3],nodeVal[3],nodeVal[3]);
		if(NULL == fgets(line, sizeof(line), famira)){
			printf("Reached end of amira file after finding element components\n");
			return -8;
		}
	}

	//print footer
	fprintf(fout,"*END\n");

	printf("Finished printing to k file.\n");
	return 0;
}

