// amiraToOff.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
	printf("Converts an amira hypermesh file to an off file\n");
	printf("Usage: amiraToOff [amira mesh file (.hmascii)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.off)> = output off file name; default 'out.off'\n");
	printf("  -print              = print debugging progress messages to screen\n");
	printf("  -t <triangles (int)>= number of triangles to convert: default 'all present'\n");
	printf("  -v <vertices (int)> = number of vertices  to convert: default 'all present'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fmesh, *foff;
	bool doPrint=false;
	int vertices=0, triangles=0, edges=0;

	fmesh = fail_or_open(argv[1], "rt");
	FileCloser fc1(fmesh);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            foff	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
		}else if(0==strcmp(argv[i],"-t")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&triangles)){
				printf("entered invalid number for triangels");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-v")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&vertices)){
				printf("entered invalid number for vertices");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	if(noOutFile){
		foff	= fail_or_open("out.off", "wt");
		printf("writing to file : out.off\n");
	}
	FileCloser fc2(foff);

	printf("converting from file: %s\n\n",argv[1]);

	//initalizing variables to use.
	char line[1000];
	char word[20];
	
	if(NULL==fgets(line, sizeof(line), fmesh) ){
		printf("Failed to read first line of .hmascii file\n");
		return -3;
	}

	// finding number of Vertices from .hmascii file
	if(vertices==0){
		while (NULL==strstr(line,"*node")){
			if(NULL==fgets(line, sizeof(line), fmesh)){
				printf("Reached end of file before finding '*node'\n");
				return -4;
			}
		}
		while (NULL!=strstr(line,"*node")){
			sscanf(line,"%c%c%c%c%c%c%d",&word[0],&word[1],&word[2],&word[3],&word[4],&word[5],&vertices);
			fgets(line,sizeof(line),fmesh);
		}
	}
	// finding number of Triangels from .hmascii file
	if(triangles==0){
		while (NULL==strstr(line,"*tri")){
			if(NULL == fgets(line, sizeof(line), fmesh)){
				printf("Reached end of file before finding '*tri'\n");
				return -5;
			}
		}
		while (NULL!=strstr(line,"*tri")){
			fgets(line, sizeof(line), fmesh);
			sscanf(line,"%c%c%c%c%c%c%d",&word[0],&word[1],&word[2],&word[3],&word[4],&word[5],&triangles);
		}
	}

	// printing header to .off filer
	edges = triangles*3;

	printf("vertices: %d\n",vertices);
	printf("triangles: %d\n",triangles);
	printf("edges: %d\n",edges);

	fprintf(foff, "OFF\n");
	fprintf(foff, "%d %d %d\n", vertices, triangles, edges);
	if(doPrint){
		printf("printing header line 1/2    : OFF\n");
		printf("printing header line 2/2    : %d %d %d\n",vertices, triangles, edges);
	}

	//finding verticies in .hmascii file
	rewind(fmesh);
	while (NULL==strstr(line,"*node")){
		if(NULL == fgets(line, sizeof(line), fmesh)){
			printf("Reached end of file before finding '*node'\n");
			return -6;
		}
	}

	//printing vertices to .off file
	int nodeNum=0;
	double nodeCord[3]={0.0};

	for(int j=0;j<vertices;j++){
		sscanf(line,"%c%c%c%c%c%c%d%c%lf%c%lf%c%lf"
			,&word[0],&word[1],&word[2],&word[3],&word[4],&word[5]
			,&nodeNum,&word[6]
			,&nodeCord[0],&word[7]
			,&nodeCord[1],&word[8]
			,&nodeCord[2]);
		fprintf(foff,"%.6lf %.6lf %.6lf\n",nodeCord[0],nodeCord[1],nodeCord[2]);
		if(NULL == fgets(line, sizeof(line), fmesh)){
			printf("Reached end of file before finding all verticies\n");
			return -7;
		}
	}

	//finding triangles in .hmascii file
	rewind(fmesh);
	while (NULL==strstr(line,"*tri")){
		if(NULL == fgets(line, sizeof(line), fmesh)){
			printf("Reached end of file before finding '*tri'\n");
			return -8;
		}
	}

	//printing triangles to .off file
	int triNum=0, triMat=0;
	int triVal[3]={0};

	for(int k=0;k<triangles;k++){
		sscanf(line,"%c%c%c%c%c%c%d%c%d%c%d%c%d%c%d"
			,&word[0],&word[1],&word[2],&word[3],&word[4],&word[5]
			,&triNum,&word[6]
			,&triMat,&word[8]
			,&triVal[0],&word[9]
			,&triVal[1],&word[10]
			,&triVal[2]);
			fprintf(foff,"3 %d %d %d\n",triVal[0]-1,triVal[1]-1,triVal[2]-1);
		if(NULL == fgets(line, sizeof(line), fmesh)){
			printf("Reached end of file before finding all triangles\n");
			return -9;
		}
	}

	printf("Finished printing to off file.\n");
	return 0;
}

