// changeProperties.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
	printf("Changes properties (numbering and material values) of a file in lsdyna .k format\n");
	printf("Usage: changeProperties [lsdyna file (.k)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -enum <elementNum (int)> = start numbering elements with this number; default 'noChange'\n");
	printf("  -eval <elementVal (int)> = material value to use for elements; defalut 'noChange'\n");
	printf("  -nnum <nodeNum (int)>    = add this number to node#, element connectivity #s, and shell connectivity #s; defalut 'noChange'\n");
	printf("  -o <outfile (.k)>        = output file name (.k); default 'out.k'\n");
	printf("  -snum <shellNum (int)>   = start numbering shells with this number; default 'noChange'\n");
	printf("  -sval <shellVal (int)>   = material value to use for shells; defalut 'noChange'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	printf("*** Caution: Blank lines in .k file may produce unexpected results ***\n\n");

	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fin,*fout;
	int elementNum=0,elementVal=0,nodeNum=0,shellNum=0,shellVal=0;
	bool doEnum=false,doEval=false,doNnum=false,doSnum=false,doSval=false;

	fin = fail_or_open(argv[1], "rt");
	FileCloser fc1(fin);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-enum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&elementNum)){
				printf("entered invalid number for initial element index\n");
				usage();
				return -2;
			}
			else{
				printf("start elements with : %d\n",elementNum);
				doEnum=true;
				i++;
			}
		}else if(0==strcmp(argv[i],"-eval")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&elementVal)){
				printf("entered invalid number for new element value\n");
				usage();
				return -2;
			}
			else{
				printf("assinging element value : %d\n",elementVal);
				doEval=true;
				i++;
			}
		}else if(0==strcmp(argv[i],"-nnum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeNum)){
				printf("entered invalid number for adding to node index\n");
				usage();
				return -2;
			}
			else{
				printf("adding to nodes value : %d\n",nodeNum);
				doNnum=true;
				i++;
			}
		}else if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-snum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&shellNum)){
				printf("entered invalid number for initial shell index\n");
				usage();
				return -2;
			}
			else{
				printf("start shells with : %d\n",shellNum);
				doSnum=true;
				i++;
			}
		}else if(0==strcmp(argv[i],"-sval")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&shellVal)){
				printf("entered invalid number for new shell value\n");
				usage();
				return -2;
			}
			else{
				printf("assinging shell value : %d\n",shellVal);
				doSval=true;
				i++;
			}
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	FileCloser fc2(fout);

	printf("converting from file : %s\n\n",argv[1]);

	//scan every line in fin file
	char line[1000],printline[1000];
	int tempInt[10]={0};
	double tempDbl[4]={0.0};
	bool isNode=false,isElement=false,isShell=false,isShellThick=false;

	while(NULL!=fgets(line,sizeof(line),fin)){
		strcpy(printline,line);

		//assign values to print string
		if(strstr(line,"$")){
		}else if(strstr(line,"*")){
			isNode=false;
			isElement=false;
			isShell=false;
		}else if(isNode){
			if(1!=sscanf(line,"%d",&tempInt[0])){
				isNode=false;
			}else{
				//scan tab format
				if(3==sscanf(line,"%*d%lf%lf%lf",&tempDbl[0],&tempDbl[1],&tempDbl[2]))
				{}
				//scan comma format
				else if(3!=sscanf(line,"%*d%*c%lf%*c%lf%*c%lf",&tempDbl[0],&tempDbl[1],&tempDbl[2])){
					isNode=false;
				}
				if(doNnum)
					tempInt[0]+=nodeNum;
				//print in tab format
				sprintf(printline,"%8d%16.8lf%16.8lf%16.8lf\n",tempInt[0],tempDbl[0],tempDbl[1],tempDbl[2]);			
			}
		}else if(isElement){
			//scan tab format
			if(10==sscanf(line,"%d%d%d%d%d%d%d%d%d%d"
				,&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5],&tempInt[6],&tempInt[7],&tempInt[8],&tempInt[9]))
			{}
			//scan comma format
			else if(10!=sscanf(line,"%d%*c%d%*c%d%*c%d%*c%d%*c%d%*c%d%*c%d%*c%d%*c%d"
				,&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5],&tempInt[6],&tempInt[7],&tempInt[8],&tempInt[9]))
			{
				isElement=false;
			}
			if(doEnum){
				tempInt[0]=elementNum;
				elementNum++;
			}
			if(doEval)
				tempInt[1]=elementVal;
			if(doNnum){
				for(int i=2;i<10;i++){
					tempInt[i]+=nodeNum;
				}
			}
			//print in tab format
			sprintf(printline,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n"
				,tempInt[0],tempInt[1],tempInt[2],tempInt[3],tempInt[4],tempInt[5],tempInt[6],tempInt[7],tempInt[8],tempInt[9]);
		}else if(isShell){
			if(isShellThick){
				isShellThick=false;
				//scan tab format
				if(4==sscanf(line,"%lf%lf%lf%lf",&tempDbl[0],&tempDbl[1],&tempDbl[2],&tempDbl[3]))
				{}
				//scan comma format
				else if(4!=sscanf(line,"%lf%*c%lf%*c%lf%*c%lf",&tempDbl[0],&tempDbl[1],&tempDbl[2],&tempDbl[3])){
					isShell=false;
				}
				//print in tab format
				sprintf(printline,"%16.8lf%16.8lf%16.8lf%16.8lf\n",tempDbl[0],tempDbl[1],tempDbl[2],tempDbl[3]);			
			}else{
				isShellThick=true;
				//scan tab format
				if(6==sscanf(line,"%d%d%d%d%d%d"
					,&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5]))
				{}
				//scan comma format
				else if(6!=sscanf(line,"%d%*c%d%*c%d%*c%d%*c%d%*c%d"
					,&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5]))
				{
					isShell=false;
				}
				if(doSnum){
					tempInt[0]=shellNum;
					shellNum++;
				}
				if(doSval)
					tempInt[1]=shellVal;
				if(doNnum){
					for(int j=2;j<6;j++){
						tempInt[j]+=nodeNum;
					}
				}
				//print in tab format
				sprintf(printline,"%8d%8d%8d%8d%8d%8d\n"
					,tempInt[0],tempInt[1],tempInt[2],tempInt[3],tempInt[4],tempInt[5]);
			}
		}

		//check for tags
		if(strstr(line,"*NODE")){
			isNode=true;
		}else if(strstr(line,"*ELEMENT_SOLID")){
			isElement=true;
		}else if(strstr(line,"*ELEMENT_SHELL_THICKNESS")){
			isShell=true;
		}

		//print string to fout file
		fprintf(fout,"%s",printline);
	}

	printf("Finished writing to file.\n");
	return 0;
}

