// changeShellNormals.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 2/1/05

#pragma once
#include "stdafx.h"
#include <iostream>

// USAGE  prints usage message to screen
void usage()
{
	printf("Changes normals for all Element Shells and their Thicknesses given in a lsdyna .k file.  For example, a shell with connectivity 1 2 3 3 will be printed with connectivity 3 2 1 1.\n");
	printf("Usage: changeShellNormals [lsdyna file (.k)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.k)>        = output file name (.k); default 'out.k'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{

	printf("\n");
	
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fin,*fout;
	
	fin = fail_or_open(argv[1], "rt");
	FileCloser fc1(fin);

	//get and save options from the user
	bool noOutFile=true;
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	FileCloser fc2(fout);

	printf("converting from file : %s\n\n",argv[1]);


	//scan every line in fin file
	char line[1000];
	int tempInt[6]={0};
	double tempDbl[4]={0.0};
	bool isShell=false;

	while(NULL!=fgets(line,sizeof(line),fin)){
		if(strstr(line,"$")){
			fprintf(fout,"%s",line);
		}else if(isShell&&strstr(line,"*")){
			isShell=false;
			fprintf(fout,"%s",line);
		}else if(isShell){
			if(6!=sscanf(line,"%d%d%d%d%d%d",&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5])){
				printf(" Error reading in shell connectivity values... \n");
				return -3;
			}
			fprintf(fout,"%8d%8d%8d%8d%8d%8d\n",tempInt[0],tempInt[1],tempInt[4],tempInt[3],tempInt[2],tempInt[2]);
			if(NULL==fgets(line,sizeof(line),fin)){
				printf(" Error scanning thickness values line\n");
				return -4;
			}
			if(4!=sscanf(line,"%lf%lf%lf%lf",&tempDbl[0],&tempDbl[1],&tempDbl[2],&tempDbl[3])){
				printf(" Error reading in shell thickness values... \n");
				return -5;
			}
			fprintf(fout,"%16.8lf%16.8lf%16.8lf%16.8lf\n",tempDbl[2],tempDbl[1],tempDbl[0],tempDbl[0]);
		}else if(strstr(line,"*ELEMENT_SHELL_THICKNESS")){
			isShell=true;
			fprintf(fout,"%s",line);
		}else{
			fprintf(fout,"%s",line);
		}
	}

	printf("Finished reversing all shell normals\n");
	return 0;
}

