// changeShells.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "../nodeReader/nodeReader.cpp"

// USAGE  prints usage message to screen
void usage()
{
   	printf("Changes shell properties of a lsdyna .k file for given shell indexes\n");
	printf("Usage: changeShells [lsdyna shell file (.k)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -nfile <fnodes (.shells)>  = trugrid output file list of shell indexes to change; default 'changes all shells'\n");
	printf("  -o <outfile (.k)>          = output file name (.k); default 'out.k'\n");
	printf("  -print                     = print debugging progress messages to screen\n");
	printf("  -t <newThickness (double)> = thickness value to assign to given shells; default 'no change'\n");
	printf("  -v <newValue (int)>        = part number value to assign to given shells; default 'no change'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fk,*fnodes=NULL,*fout=NULL;
	bool doPrint=false, doThick=false, doValue=false;
	double newThickness=0.0;
	int newValue=0;

	fk = fail_or_open(argv[1],"rt");
	FileCloser fc1(fk);

	//get and save options from the user
	bool useOut=false;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-nfile")){
	        fnodes	= fail_or_open(argv[i+1], "rt");
			printf("useing only shells in file : %s\n",argv[i+1]);
			i++;
		}else if(0==strcmp(argv[i],"-o")){
			fout = fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			useOut=true;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
		}else if(0==strcmp(argv[i],"-t")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&newThickness)){
				printf("entered invalid number for new thickness");
				usage();
				return -2;
			}
			else{
				printf("assigning thickness : %0.8f\n",newThickness);
				doThick=true;
				i++;
			}
		}else if(0==strcmp(argv[i],"-v")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&newValue)){
				printf("entered invalid number for new material number");
				usage();
				return -2;
			}
			else{
				printf("assigning mat value : %d\n",newValue);
				doValue=true;
				i++;
			}
		}
	}
	if(!useOut){
        fout = fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	FileCloser fc2(fnodes),fc3(fout);

	printf("Reading shell values from: %s\n\n",argv[1]);

	//find beginning of shell in fout (.k)
	char line[1000];
	
	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of fout (.k) file\n");
		return -3;
	}
	while(NULL==strstr(line,"*ELEMENT_SHELL_THICKNESS")){
		fprintf(fout,"%s",line);
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fout (.k) file before finding shells\n");
			return -4;
		}
	}
	fprintf(fout,"%s",line);
	
	//find and replace each node value from fnodes file
	bool useFile=true;
	nodeReader nr;
	int nxtnode=0;
	if(fnodes==NULL){
		useFile=false;
	}else{
		nr.setFile(fnodes);
		nxtnode=nr.getNext();
	}
	bool foundNode=false;
	int	tempInt[6]={0};
	
	//find and replace nxtnode in fout (.k) file
	while(NULL!=fgets(line,sizeof(line),fk)){
		if(strstr(line,"$")){
			fprintf(fout,"%s",line);
		}else if(strstr(line,"*")){
			break;
		}else if(6==sscanf(line,"%d%d%d%d%d%d"
			,&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5]))
		{
			if(tempInt[0]==nxtnode||!useFile){
				foundNode=true;
				//replace material value
				if(doValue){
					tempInt[1]=newValue;
					if(doPrint){
						printf("replaced shell material of shell#: %d\n",tempInt[0]);
					}
				}
				fprintf(fout,"%8d%8d%8d%8d%8d%8d\n"
						,tempInt[0],tempInt[1],tempInt[2],tempInt[3],tempInt[4],tempInt[5]);
				//replace thickness values
				fgets(line,sizeof(line),fk);
				if(strstr(line,"$")||strstr(line,"*")){
					fprintf(fout,"%s",line);
					printf("Failed to scan thickness values.\n");
					return -7;
				}
				if(doThick){
					fprintf(fout,"%16.8lf%16.8lf%16.8lf%16.8lf\n",newThickness,newThickness,newThickness,newThickness);
					if(doPrint){
						printf("replaced thickness of shell#: %d\n",tempInt[0]);
					}
				}else{
					fprintf(fout,"%s",line);
				}
			}else{
				fprintf(fout,"%s",line);
				fgets(line,sizeof(line),fk);
				fprintf(fout,"%s",line);
			}

		}else{
			printf("Error scanning shell index\n");
			return -8;
		}
        if(foundNode){
			foundNode=false;
            if(useFile) nxtnode=nr.getNext();
		}
	}//end while-finding all nodes in file
	printf("Reached end of nodes file.\n");
	
	fprintf(fout,"%s",line);
	while(NULL!=fgets(line,sizeof(line),fk)){
		fprintf(fout,"%s",line);
	}
	printf("Reached end of input file.\n");

	return 0;

}
