// extractSurface.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 1/25/05

#pragma once
#include "stdafx.h"
#include <iostream>

#include "../nodeReader/nodeReader.cpp"


// USAGE  prints usage message to screen
void usage()
{
    printf("Extracts the elements of given nodes from a 3d lsdyna .k file\n");
	printf("Usage: extractSurface [input mesh (.k)] [input nodes (.nodes)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -nnum <nodeNum (int)>       = number of nodes in input nodes file; default 'scans file to determine number'\n");
	printf("  -o <fout (.k)>              = output file name (.k); default 'out.k'\n");
	printf("  -origin <ox oy oz (double)> = coordinates for additional origin node; default '0.0 0.0 0.0'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

// Code for BinarySearch
int BinarySearch(int* IntArray, int Low, int High, int Target)
{

   int Mid, Difference;

   while (Low <= High)
   {
      Mid = (Low + High) / 2;
      Difference = IntArray[Mid] - Target;

      if (Difference == 0)         // IntArray[Mid] == Target
         return Mid;
      else if (Difference < 0)     // IntArray[Mid] < Target
         Low = Mid + 1;
      else
         High = Mid - 1;
   }

   return -1;   // If reach here, Target was not found.
}
int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fk, *fnodes=NULL, *fout=NULL;
	int nodeNum=0;
	double ox=0.0,oy=0.0,oz=0.0;

	fk = fail_or_open(argv[1], "rt");
	fnodes = fail_or_open(argv[2], "rt");
	FileCloser fc1(fk),fc2(fnodes);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=3;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
	        fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-origin")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&ox)){
				printf("entered invalid x for origin");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&oy)){
				printf("entered invalid y for origin");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+3]);
			if(1!=sscanf(toConvert,"%lf",&oz)){
				printf("entered invalid z for origin");
				usage();
				return -2;
			}
			else
				i+=3;
		}else if(0==strcmp(argv[i],"-nnum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeNum)){
				printf("entered invalid number of nodes");
				usage();
				return -2;
			}
			else{
				i++;
				printf("Number of nodes: %d\n",nodeNum);
			}
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}

	FileCloser fc3(fout);
	printf("origin coordinates: %lf %lf %lf\n",ox,oy,oz);
	printf("Reading from input file: %s\n",argv[1]);
	printf("Extracting nodes from file: %s\n\n",argv[2]);

	//start nodeReader
	nodeReader nr;
	nr.setFile(fnodes);
	int nextNode=nr.getNext();
	    
	//Scan through to find out how many nodes there are.
	if(nodeNum==0){
		while(nextNode!=0){
			nodeNum++;
			nextNode=nr.getNext();
		}
		printf("Number of nodes found: %d\n",nodeNum);
		
		//restart nodeReader
		fclose(fnodes);
		fnodes = fail_or_open(argv[2], "rt");
		nr.setFile(fnodes);
		nextNode=nr.getNext();
	}

	//find nodes in .k file
	char line[1000];
	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of fk (.k) file\n");
		return -3;
	}
	while(NULL==strstr(line,"*NODE")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding nodes\n");
			return -4;
		}
	}

	//print header to output
	fprintf(fout,"*KEYWORD\n");
	
	//scan through each given node number
	int* nodes;
	nodes=new int[nodeNum];
	int index;
	bool foundNode=false;
	int tempInt[8]={0};
	double tempDbl[3]={0.0};

	fprintf(fout,"*NODE\n");
	for(index=0;index<nodeNum;index++){
        //save each node number to array with new index
		nodes[index]=nextNode;
		nextNode=nr.getNext();
		if(nodes[index]==0)	break;

		//find node with current index
		while(!foundNode&&NULL!=fgets(line,sizeof(line),fk)){
			if(strstr(line,"$"))
			{}
			else if(strstr(line,"*"))
				break;
			else if(4!=sscanf(line,"%d%lf%lf%lf",&tempInt[0],&tempDbl[0],&tempDbl[1],&tempDbl[2])){
				printf("Error reading node number.\n");
				return -5;
			}else if(tempInt[0]==nodes[index])
				foundNode=true;
		}
		
		//print node
		if(!foundNode){
			printf("Unable to find node %d\n",nodes[index]);
		}
		else
			fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",index+1,tempDbl[0],tempDbl[1],tempDbl[2]);

		//reset foundNode
		foundNode=false;
	}
	if(nodeNum!=(index)){
		nodeNum=index;
		printf("Number of nodes found: %d\n",nodeNum);
		
	}
	//Print additonal origin node
	fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",nodeNum+1,ox,oy,oz);


	
	//find elements in .k file
	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of element file (fe.k)\n");
		return false;
	}
	while(NULL==strstr(line,"*ELEMENT_")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of element file before finding *ELEMENT_SOLID\n");
			return false;
		}
	}
	
	//for solid elements
	int elemNum=0;
	int foundNum=0;
	if(NULL!=strstr(line,"*ELEMENT_SOLID")){
		fprintf(fout,"*ELEMENT_SOLID\n");

		//scan through each element
		while(NULL!=fgets(line,sizeof(line),fk)){
			if(strstr(line,"$"))
			{}
			else if(strstr(line,"*"))
				break;
			else if(8!=sscanf(line,"%*d%*d%d%d%d%d%d%d%d%d",&tempInt[0],&tempInt[1],&tempInt[2],&tempInt[3],&tempInt[4],&tempInt[5],&tempInt[6],&tempInt[7])){
					printf("Error reading element values.\n");
					return false;
			}else{
				//for all connectivity
				for(int j=0;j<8;j++){
					//search for number in array (use some function, see help)
					index=BinarySearch(nodes,0,nodeNum-1,tempInt[j]);
					//save new index if found
					if(index>=0){
						foundNum++;
						tempInt[j]=index+1;
					}else
						tempInt[j]=nodeNum+1;
				}
                //if 4 nodes were found, print the element
				if(foundNum==4){
					elemNum++;
					fprintf(fout,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n"
						,elemNum,1,tempInt[0],tempInt[1],tempInt[2],tempInt[3],tempInt[4],tempInt[5],tempInt[6],tempInt[7]);
				}
				foundNum=0;
			}
		}
		printf("Number of elements found: %d\n",elemNum);
	}


	fprintf(fout,"*END\n");
	
	delete [] nodes;
	return 0;
}

