// circleMapper.cpp: implementation of the circleMapper class.

#pragma once
#include "circleMapper.h"

circleMapper::circleMapper(double slp, double intcpt, int xresolution, int yresolution)
{
	slope=slp,intercept=intcpt;
	xres=xresolution,yres=yresolution;
	numbins=xres*yres;
	binvalues=new double[numbins];
}

circleMapper::~circleMapper()
{
	slope=0.0,intercept=0.0;
	xres=0,yres=0;
	numbins=0;
	delete [] binvalues; binvalues=NULL;
}

bool circleMapper::loadImage(const char* imageFileName)
{
	//open file for reading binary raw data.
	FILE* fi = fopen(imageFileName, "rb");
	if(fi == NULL){
		printf(" failed to read raw input file ... exiting\n");
		return false;
	}

	//read in char sized data
	int numFound;
	unsigned char* charVals = new unsigned char[xres*yres];
	
	numFound=fread(charVals, sizeof(char), xres*yres, fi);
	fclose(fi);
	printf(" Found %d values in image file.\n",numFound);

	//convert char data to double and save values
	//  charRange: 0(black) to 256(white)=256
	//  Pressure conversion:
	//   Invert-- 256-charVals
	//   Apply Line-- Invert*slope+intercept
	int charIdx=0;
	int binNum=0;
	
	// the raw image reads from top left to top right to bottom.
	// we want the result to read from bottom left to bottom right to top.
	// the charIdx,i,j variables deal with this.
	for(int i=yres-1;i>=0;i--){
		for(int j=xres-1;j>=0;j--)
		{		
			charIdx=i*xres+(xres-1)-j;
			binvalues[binNum]=(256-(int)charVals[charIdx])*slope+intercept;
			binNum++;
		}
	}

	return true;
}

bool circleMapper::printGridToK(FILE* fk,double maxPressure,int nodenum)
{

	//print connectivity
	int corner=0;
	int c1=0,c2=0,c3=0,c4=0;
	int	binNum=0;
	
	fprintf(fk,"$Grid Connectivity with Pressuress\n");
	fprintf(fk,"*ELEMENT_SHELL_THICKNESS\n");
	for(int n=0;n<yres-1;n++){
		for(int m=0;m<xres-1;m++){
			c1=corner+1;
			c2=corner+2;
			c3=corner+xres+2;
			c4=corner+xres+1;
			if(binvalues[c1-1]>=maxPressure||binvalues[c2-1]>=maxPressure||binvalues[c3-1]>=maxPressure||binvalues[c4-1]>=maxPressure)
				fprintf(fk,"%8d%8d%8d%8d%8d%8d\n",binNum+1,1,c1+nodenum,c2+nodenum,c3+nodenum,c4+nodenum);
			else
				fprintf(fk,"%8d%8d%8d%8d%8d%8d\n",binNum+1,2,c1+nodenum,c2+nodenum,c3+nodenum,c4+nodenum);
			fprintf(fk,"%16.8lf%16.8lf%16.8lf%16.8lf\n",binvalues[c1-1],binvalues[c2-1],binvalues[c3-1],binvalues[c4-1]);
			binNum++;
			corner++;
		}
		corner++;
	}

	return true;
}

bool circleMapper::printNodesToK(FILE* fk,int nodeStart)
{
	fprintf(fk,"$Grid Nodes (centered around the origin)\n");
	fprintf(fk,"*NODE\n");
	double xstart=(xres-1)/2.0,ystart=(yres-1)/2.0;
	int nodeNum=nodeStart;

	for(double i=-ystart;i<=ystart;i++){
		for(double j=-xstart;j<=xstart;j++){
            fprintf(fk,"%8d%16.8lf%16.8lf%16.8lf\n",++nodeNum,j,i,0.0);
		}
	}

	return true;
}
