// fujiToImage.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 1/07/05

#include "stdafx.h"

#include <iostream>
#include "circleMapper.cpp"


// USAGE  prints usage message to screen
void usage()
{
    printf("Converts a 2D 8-bit image into a 2D mapped .k image\n");
	printf("Usage: fujiToImage [input file (.raw)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -maxp <maxPressure (double)>\t= max value of presure output to be included in part 2; default '3.5'\n");
	printf("  -nstart <nodeStart (int)>\t\t= value to add when printing node numbers; default '0'\n");
	printf("  -o <outfile (.k)>\t\t\t= output file name (.k); default 'out.nike'\n");
	printf("  -pmap <slope (double) intercept (double)>\t= slope and intercept to use to map the inverted raw pixel value (256-value) to pressure output (usually MPa); default '0.0178 0.3097'\n");
	printf("  -res <xres yres (double)>\t\t\t= pixel resolution of input image; default '256 256'\n");
    exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	char *imageFile = argv[1];
	FILE *fout=NULL;
	double maxPressure=3.5;
	int nodeStart=0;
	double slope=0.0178,intercept=0.3097;
	int xres=256,yres=256;

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-maxp")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&maxPressure)){
				printf("entered invalid value for max pressure of part 2");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-nstart")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeStart)){
				printf("entered invalid value for initial node number");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-o")){
	        fout	= fail_or_open(argv[i+1], "at");
			printf("appending output to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-pmap")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&slope)){
				printf("entered invalid slope for pressure mapping");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&intercept)){
				printf("entered invalid intercept for pressure mapping");
				usage();
				return -2;
			}
			else
				i+=2;
		}else if(0==strcmp(argv[i],"-res")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&xres)){
				printf("entered invalid x for image resolution");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%d",&yres)){
				printf("entered invalid y for image resolution");
				usage();
				return -2;
			}
			else
				i+=2;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "at");
		printf("appending output to file : out.k\n");
	}
	FileCloser fc6(fout);
	printf("max pressure : %lf\n",maxPressure);
	printf("starting node : %d\n",nodeStart);
	printf("pressure map : %lf slope %lf intercept\n",slope,intercept);
	printf("resolution : %d x %d\n",xres,yres);
	printf("Reading image data from file: %s\n\n",imageFile);



	// Initialize a circleMapper
	circleMapper cmap(slope,intercept,xres,yres);
	// Print header to output file
	fprintf(fout,"*KEYWORD\n");

	// Get image values from (.raw) file
	printf("\nLoading image data ...\n");
	if(!cmap.loadImage(imageFile)){
		printf("Failure loading image values from .raw file\n");
		return -3;
	}
    
	if(true){
		printf("\nPrinting Nodes to grid (appending)...\n");
		if(!cmap.printNodesToK(fout,nodeStart)){
			printf("Failed to print grid to file\n");
			return -10;
		}
	}

	if(true){
		printf("\nPrinting Element Shells to grid with current pressures (appending)...\n");
		if(!cmap.printGridToK(fout,maxPressure,nodeStart)){
			printf("Failed to print grid to file\n");
			return -10;
		}
	}


	fprintf(fout,"*END\n");
	return 0;
}

