// kToNikeNodes.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#pragma once
#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
    printf("Converts a lsdyna .k file to a nike file\n");
	printf("Usage: kToNike [input file (.k)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.k)>          = output file name (.k); default 'out.nike'\n");
	printf("  -val <newVal (int)>        = material value to use for new format; defalut 'uses values for tc rc given after node coordinates'\n");
    exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fk, *fout;
	bool useNewVal=false;
	int newVal=0;

	fk = fail_or_open(argv[1], "rt");
	FileCloser fc1(fk);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
	        fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-val")){
			useNewVal=true;
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&newVal)){
				printf("entered invalid number for new value");
				usage();
				return -2;
			}
			else{
				printf("assigning material value: %d\n",newVal);
				i++;
			}
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.nike", "wt");
		printf("writing to file : out.nike\n");
	}
	FileCloser fc2(fout);

	printf("Reading from file: %s\n\n",argv[1]);

	//find beginning of nodes in fk (.k)
    char line[1000];

	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of fk (.k) file\n");
		return -3;
	}
	while (NULL==strstr(line,"*NODE")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding '*NODE'\n");
			return -4;
		}
	}
	fgets(line,sizeof(line),fk);
	while(NULL!=strstr(line,"$")){
		fgets(line,sizeof(line),fk);
	}

	//copy nodes and print to fout
	int geti[3],someInt;
	double getd[3];
	bool isCommaFormat=false,isSpaceFormat=false;

	//scan first node line and determine format
	if(!useNewVal){
		if(6==sscanf(line,"%d%*c%lf%*c%lf%*c%lf%*c%d%*c%d",&geti[0],&getd[0],&getd[1],&getd[2],&geti[1],&geti[2])){
			isCommaFormat=true;
		} else if (6==sscanf(line,"%d%lf%lf%lf%d%d",&geti[0],&getd[0],&getd[1],&getd[2],&geti[1],&geti[2])){
			isSpaceFormat=true;
		} else {
			printf("unrecognized node format found.  Expected: id x y z tc rc \n");
			return -5;
		}
	}else{
		if(4==sscanf(line,"%d%*c%lf%*c%lf%*c%lf",&geti[0],&getd[0],&getd[1],&getd[2])){
			isCommaFormat=true;
		} else if (4==sscanf(line,"%d%lf%lf%lf",&geti[0],&getd[0],&getd[1],&getd[2])){
			isSpaceFormat=true;
		} else {
			printf("unrecognized node format found. Expected: id x y z \n");
			return -5;
		}
	}

	//scan node using correct format, print node, and get next line
	while(1==sscanf(line,"%d",&someInt)){
		if(isCommaFormat){
			if(!useNewVal){
				if(6!=sscanf(line,"%d%*c%lf%*c%lf%*c%lf%*c%d%*c%d",&geti[0],&getd[0],&getd[1],&getd[2],&geti[1],&geti[2])){
					printf("failed to read all nodes in comma format.\n");
					return-6;
				}
				fprintf(fout,"%8d%5d%20.12E%20.12E%20.12E%5d\n",geti[0],geti[1],getd[0],getd[1],getd[2],geti[2]);
			}else{
				if(4!=sscanf(line,"%d%*c%lf%*c%lf%*c%lf",&geti[0],&getd[0],&getd[1],&getd[2])){
					printf("failed to read all nodes in comma format.\n");
					return-6;
				}
				geti[1]=geti[2]=newVal;
				fprintf(fout,"%8d%5d%20.12E%20.12E%20.12E%5d\n",geti[0],geti[1],getd[0],getd[1],getd[2],geti[2]);
			}
		}else if(isSpaceFormat){
			if(!useNewVal){
				if(6!=sscanf(line,"%d%lf%lf%lf%d%d",&geti[0],&getd[0],&getd[1],&getd[2],&geti[1],&geti[2])){
					printf("failed to read all nodes in space format.\n");
					return-7;
				}
				fprintf(fout,"%8d%5d%20.12E%20.12E%20.12E%5d\n",geti[0],newVal,getd[0],getd[1],getd[2],newVal);
			}else{
				if(4!=sscanf(line,"%d%lf%lf%lf",&geti[0],&getd[0],&getd[1],&getd[2])){
					printf("failed to read all nodes in space format.\n");
					return-7;
				}
				geti[1]=geti[2]=newVal;
				fprintf(fout,"%8d%5d%20.12E%20.12E%20.12E%5d\n",geti[0],newVal,getd[0],getd[1],getd[2],newVal);
			}
		}
		if(NULL==fgets(line,sizeof(line),fk)){
			printf("unable to read line\n");
			return -8;
		}
	}

	printf("Finished printing.\n");
	return 0;
}

