// makeShells.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "../nodeReader/nodeReader.cpp"

// USAGE  prints usage message to screen
void usage()
{
	printf("Creates shells with same connectivity of given numbers (nodes) in a lsdyna .k file\n"); 
	printf("Usage: makeShells [lsdyna connectivity shells file (.k)] [trugrid shell index file (.nodes)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -copy                      = copy contents of input .k file before appending the new shells\n");
	printf("  -num <shellNum (int)>      = number of shell to start printing; defalut '1'\n");
	printf("  -o <outfile (.k)>          = output file name (.k); default 'out.k'\n");
	printf("  -t <newThickness (double)> = thickness value to assign to new shells; default '1.5'\n");
	printf("  -val <shellVal (int)>      = material value to assign to new shells; defalut '3'\n");
    exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(argc<2) {
		usage();
		return -1;
	}

	FILE *fnodes, *fk, *fout;
	int shellNum=1, shellVal=3;
	bool doCopy=false;
	double newThickness=1.5;
	
	fk		= fail_or_open(argv[1], "rt");
	fnodes = fail_or_open(argv[2], "rt");
	FileCloser fc1(fnodes), fc2(fk);

	//get and save options from the user
	bool noOutFile=true;
	char toConvert[20];
	for(int i=3;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-num")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&shellNum)){
				printf("entered invalid number for new shell beginning index");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-copy")){
			doCopy=true;
		}else if(0==strcmp(argv[i],"-t")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&newThickness)){
				printf("entered invalid number for new thickness");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-val")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&shellVal)){
				printf("entered invalid number for new shell value");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	FileCloser fc3(fout);

	printf("using initial shell num : %d\n",shellNum);
	printf("assigning thickness : %16.8f\n",newThickness);
	printf("assigning material value: %d\n",shellVal);
	printf("making shells for nodes from file: %s\n",argv[2]);
	printf("using connectivity from file: %s\n\n",argv[1]);

	//copy original file before appending new shells
	char line[1000];

	if(doCopy){
		printf("printing copy of old .k file to new file\n");
		strcpy(line,"blank");
		while(NULL!=fgets(line, sizeof(line), fk)){
			if(NULL!=strstr(line,"END")){
				break;
			}
			fputs(line,fout);
		}
		rewind(fk);
	}

	//find beginning of shell in fk (.k)
	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of fk (.k) file\n");
		return -3;
	}
	while (NULL==strstr(line,"*ELEMENT_SHELL_THICKNESS")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding shells\n");
			return -4;
		}
	}
	
	//find next node in shell and appending to end of fout (.k) file
	nodeReader nr;
	nr.setFile(fnodes);
	int nxtnode=nr.getNext(),prvnode=0,beginningNumber=0;
	int toPrint[6];

	printf("appending new shells to new file\n");
	while(0!=nxtnode&&prvnode<nxtnode){
		while(beginningNumber!=nxtnode){
			if(NULL==fgets(line, sizeof(line), fk) ){
				printf("Failed to find shell # in .k file: %d\n",nxtnode);
				return -5;
			}
			if(1!=sscanf(line,"%d",&beginningNumber)){
				beginningNumber=0;
			}
		}
		if(6!=sscanf(line,"%d%d%d%d%d%d",&toPrint[0],&toPrint[1],&toPrint[2],&toPrint[3],&toPrint[4],&toPrint[5])){
			printf("Failed to read values from shell # %d in .k file\n",nxtnode);
			return -6;
		}
		fprintf(fout,"%8d%8d%8d%8d%8d%8d\n",shellNum,shellVal,toPrint[2],toPrint[3],toPrint[4],toPrint[5]);
		fprintf(fout,"%16.8f%16.8f%16.8f%16.8f\n",newThickness,newThickness,newThickness,newThickness);
        shellNum++;
		prvnode=nxtnode;
		nxtnode=nr.getNext();
	}
	if(doCopy){
        fprintf(fout,"*END\n");
	}

	printf("Finished printing new shells.\n");
	return 0;
}

