// Mesh.h: interface for the CMesh class.

#pragma once
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "ImageSet.h"

class CMesh  
{
public:
	CMesh(int elements, int nodes, int surfs, int minIntensity, int maxIntensity);
	virtual ~CMesh();

	bool ReadMesh(const char* szFileName);
	bool findModulus(CImageSet& set, double xadj, double yadj, double zadj);
	bool labelElements(const char* meshFileName, int startMaterial);
	bool printElements(const char* meshFileName, const char* newFileName, bool doCopy, int startElem);

	int size() { return m_nNodes; }


private:
	double intensityToEM(int intensity)
	{
		// solving CD equation for x when y=0 gives intensity minimum)
		if(intensity<1005)
			return 0.0;
		// CD = Calcium Equivalent Density
		//double CD = ((intensity*0.0008)-0.8037);
		// AD = Apparent Density
		//double AD = CD/0.626;
		double AD = ((intensity*0.0013)-1.2839);
        // EM = Elastic Molulus
		double EM = (pow(AD, 2.46))*2017.3;

		return EM;
	}


	int		m_nElems;	//mesh number of Elements
	int		m_nNodes;	//mesh number of Nodes
	int		m_nSurfs;	//mesh number of Surface Nodes
	int		minINTENSITY;	//minimum Intensity
	int		maxINTENSITY;	//maximum Intensity
	double	minELASTICMODULUS; //minimum Elastic Modulus for an individual node
	double	maxELASTICMODULUS; //maximum Elastic Modulus for an individual node
	float	*m_pNode;	//mesh pointer to array of Nodes
	double  *m_pEM;		//mesh pointer to array of ElasticModulus for each Node
	double	*m_eEM;		//mesh pointer to array of ElasticModulus for each Element
	int		*m_eVal;	//mesh pointer to array of MaterialValue for each Element

};
