// modulus.cpp : Defines the entry point for the console application.

// Modified by Janna Balling
// 6/04/04

#include "stdafx.h"

#pragma once
#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string>

#include "ImageSet.cpp"
#include "Mesh.cpp"

// USAGE  prints usage message to screen
void usage()
{
	printf("Computes the elastic modulus of shells in a lsdyna .k file using the raw data\n");
	printf("Usage: modulus [raw CT data file (.raw)] [mesh with shells file (.k)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -copy                         =  Copy contents of input .k file before appending changes to the shells\n");
	printf("  -fov <xyFOV (int)>            =  Length of CT data set along x/y axes in mm; default '225'\n");
	printf("  -min <minIn (int)>			=  Minimum intensity value from data to assign; defalut '1028' (from calcium density calibration scan)\n");
	printf("  -max <maxIn (int)>			=  Maximum intensity value from data to assign; defalut '1560' (from calcium density calibration scan)\n");
	printf("  -ne <NumElems (int)>          =  Number of Elements; default '0'\n");
	printf("  -nn <NumNodes (int)>          =  Number of total mesh nodes; default '0'\n");
	printf("  -ns <NumSurfNodes (int)>      =  Number of mesh nodes on surface; default '0'\n");
	printf("  -o <outfile (.k)>             =  Output file name; default 'out.k'\n");
	printf("  -slice <slices (int)>         =  Number of slices along z-axis in CT data; default '1'\n");
	printf("  -sm <startMat (int)>          =  Number of material to start material range; defalut '1'\n");
	printf("  -se <startElems (int)>        =  Number of elements to start printing; defalut '1'\n");
	printf("  -x <Xdemensions (int)>        =  Number of pixels along x-axis of CT data; default '512'\n");
	printf("  -y <Ydemensions (int)>        =  Number of pixels along y-axis of CT data; default '512'\n");
	printf("  -z <Zdemensions (int)>        =  Number of pixels along z-axis of 1 slice of CT data; default '1'\n");
	printf("  -zthick <SliceThick (double)> =  Thickness of CT data slices in mm; default '1.0'\n");
	exit(-1);
}


int main(int argc, char* argv[])
{
	printf("\n");
	if(argc<2) {
		usage();
		return -1;
	}

	char *imageFile = argv[1];
	char *meshFile = argv[2];
	bool doCopy=false;
	int xyFOV=225;
	int minIn=1028,maxIn=1560;
	int NumElems=0,NumNodes=0,NumSurfNodes=0;
	char *outFile = "out.k";
	int startMat=1,startElems=1;
	int Xdemensions=512,Ydemensions=512,Zdemensions=1;
	int slices=1;
	double SliceThick=1.0;
		
	//get and save options from the user
	char toConvert[20];
	for(int i=3;i<argc;i++){
		if(0==strcmp(argv[i],"-copy")){
			printf("copying original file and inserting new elements\n");
			doCopy=true;
		}else if(0==strcmp(argv[i],"-fov")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&xyFOV)){
				printf("entered invalid x/y Field Of View\n");
				usage();
				return -2;
			}
			else{
				i++;
			}
		}else if(0==strcmp(argv[i],"-min")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&minIn)){
				printf("entered invalid minimum intensity\n");
				usage();
				return -2;
			}
			else{
				i++;
			}
		}else if(0==strcmp(argv[i],"-max")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&maxIn)){
				printf("entered invalid maximum intensity\n");
				usage();
				return -2;
			}
			else{
				i++;
			}
		}else if(0==strcmp(argv[i],"-ne")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&NumElems)){
				printf("entered invalid number of Elements\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-nn")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&NumNodes)){
				printf("entered invalid number of Nodes\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-ns")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&NumSurfNodes)){
				printf("entered invalid number of surface nodes\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-o")){
			outFile=argv[i+1];
			i++;
		}else if(0==strcmp(argv[i],"-slice")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&slices)){
				printf("entered invalid number of slices (z-axis)\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-sm")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&startMat)){
				printf("entered invalid number to start materials\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-se")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&startElems)){
				printf("entered invalid number to start elements\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-x")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&Xdemensions)){
				printf("entered invalid number for x demension\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-y")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&Ydemensions)){
				printf("entered invalid number for y demension\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-z")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&Zdemensions)){
				printf("entered invalid number for z demension\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-zthick")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&SliceThick)){
				printf("entered invalid number of Shells\n");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	printf("Field of View        : %d\n",xyFOV);
	printf("Minimum Intensity    : %d\n",minIn);
	printf("Maximum Intensity    : %d\n",maxIn);
	printf("Number of Elements   : %d\n",NumElems);
	printf("Number of Nodes      : %d\n",NumNodes);
	printf("Number of Surf Nodes : %d\n",NumSurfNodes);
	printf("Number of Slices     : %d\n",slices);
	printf("Start Material Value : %d\n",startMat);
	printf("Start Element Number : %d\n",startElems);
	printf("X demensions         : %d\n",Xdemensions);
	printf("Y demensions         : %d\n",Ydemensions);
	printf("Z demensions         : %d\n",Zdemensions);
	printf("Slice thickness      : %f\n\n",SliceThick);
	printf("Input data file	 : %s\n",argv[1]);
	printf("Input mesh file  : %s\n",argv[2]);
	printf("Printing to file : %s\n\n",outFile);


	//running program
	CImageSet set;
	CMesh mesh(NumElems, NumNodes, NumSurfNodes, minIn, maxIn);

	printf("Reading images ...\n");
	if(!set.ReadImageSet(imageFile, Xdemensions, Ydemensions, Zdemensions, slices)){
		printf("Error reading images\n");
		return -3;
	}

	printf("Reading mesh ...\n");
	if(!mesh.ReadMesh(meshFile)){
		printf("Error reading mesh\n");
		return -4;
	}

 	printf("Finding the modulus ...\n");
	double xadj,yadj,zadj;
	xadj=(double)Xdemensions/xyFOV;
	yadj=(double)Ydemensions/xyFOV;
	zadj=(double)Zdemensions/SliceThick;
	if(!mesh.findModulus(set,xadj,yadj,zadj)){
		printf("Error doing the mapping\n");
		return -5;
	}

	printf("Labeling the elements ...\n");
	if(!mesh.labelElements(meshFile,startMat)){
		printf("Error labeling the elements\n");
		return -6;
	}

	printf("Printing the elements ...\n");
	if(!mesh.printElements(meshFile,outFile,doCopy,startElems)){
		printf("Error printing the elements\n");
		return -7;
	}

	printf("Done ...\n");
	return 0;
}


