// offToBYU.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#pragma once
#include <iostream>

// USAGE  prints usage message to screen
void usage()
{
	printf("Converts an off file to a BYU file\n");
    printf("Usage: offToBYU [input file (.off)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.byu)> = output file name; default 'out.byu'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *foff,*fbyu;

	foff = fail_or_open(argv[1], "rt");
	FileCloser fc1(foff);

	//get and save options from the user
	bool noOutFile=true;
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fbyu	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}
	}
	if(noOutFile){
		fbyu	= fail_or_open("out.byu", "wt");
		printf("writing to file : out.byu\n");
	}
	FileCloser fc2(fbyu);
	
	printf("converting from file: %s\n\n",argv[1]);


	char line[1000];
	//ignore 1st line
	if(NULL==fgets(line, sizeof(line), foff) ){
		printf("Failed to read first line of off file\n");
		return -3;
	}
	while (NULL==strstr(line,"OFF")){
		if(NULL == fgets(line, sizeof(line), foff)){
			printf("Reached end of .off file before finding 'OFF'\n");
			return -4;
		}
	}
	
	//get and print info from second line
	int nodes=0,triangles=0,edges=0;
	if(NULL==fgets(line, sizeof(line), foff) ){
		printf("Failed to read line following OFF in off file\n");
		return -5;
	}
	if(3!=sscanf(line,"%d%d%d",&nodes,&triangles,&edges)){
		printf("Failed to get values of second line of off file\n");
		return -6;
	}
	fprintf(fbyu,"1  %d  %d  %d\n",nodes,triangles,edges);
	fprintf(fbyu,"1  %d\n",triangles);
	
	//get and print node values
	double x,y,z;
	for(int k=0;k<nodes;k++){
		if(NULL==fgets(line, sizeof(line), foff) ){
			printf("Failed to get all node values from off file\n");
			return -7;
		}
		sscanf(line,"%lf%lf%lf",&x,&y,&z);
		fprintf(fbyu," %0.6e %0.6e %0.6e\n",x,y,z);
	}

	//get and print triangle values
	int a,b,c;
	for(int j=0;j<triangles;j++){
		if(NULL==fgets(line, sizeof(line), foff) ){
			printf("Failed to get all triangle values from off file\n");
			return -8;
		}
		sscanf(line,"%*d%d%d%d",&a,&b,&c);
		fprintf(fbyu,"%d %d %d\n",a+1,b+1,-(c+1));
	}

	printf("Done ... \n");
	return 0;
}

