// parseNormals.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 4/27/04

#include "stdafx.h"

#pragma once
#include <iostream>
using namespace std;
#include <stdio.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
	printf("Extracts normals values from a .vtk file (output of normals.tcl)\n");
	printf("Usage: parseNormals [vtk normals input file (*.vtk)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.normals)> = normals values output file name; default 'out.normals'\n");
	printf("  -print                  = print debugging progress messages to screen\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fvtk, *fout;
	bool doPrint=false;

	fvtk = fail_or_open(argv[1], "rt");
	FileCloser fc1(fvtk);

	//get and save options from the user
	bool noOutFile=true;
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.normals", "wt");
		printf("writing to file : out.normals\n");
	}
	FileCloser fc2(fout);
	
	printf("Converting from file: %s\n\n",argv[1]);


	// finding NORMALS (line before) in .vtk file
	char line[40];
	char value1[20],value2[20],value3[20];
	int normals=0;

	if(NULL==fgets(line, sizeof(line), fvtk) ){
		printf("Failed to read first line of .vtk file\n");
		return -3;
	} 
	while (NULL==strstr(line,"NORMALS")){
		if(NULL == fgets(line, sizeof(line), fvtk)){
			printf("Reached end of file before finding 'NORMALS'\n");
			return -4;
		}
	}
	if(doPrint)
		printf("found line: %s\n",line);

	// finding and printing normal values from .vtk file
	while (EOF!=fscanf(fvtk,"%s %s %s",value1,value2,value3)){
		normals++;	
		if(doPrint){
			printf("Printing line: %s %s %s\n",value1, value2, value3);
		}
		fprintf(fout,"%s %s %s\n",value1, value2, value3);
	}

	printf("normals found : %d\n",normals);
	printf("finished printing to file.\n");
	return 0;
}

