// printNodes.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#pragma once
#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "../nodeReader/nodeReader.cpp"

// USAGE  prints usage message to screen
void usage()
{
	printf("Expands and prints a list of nodes from a truegrid output list\n");
	printf("Usage: printNodes [trugrid nodes file (.nodes)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.nodes)> = output nodes file name; default 'out.nodes'\n");
	printf("  -print                = print debugging progress messages to screen\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fnodes, *fout;
	bool doPrint=false;
	
	fnodes = fail_or_open(argv[1], "rt");
	FileCloser fc1(fnodes);

	//get and save options from the user
	bool noOutFile=true;
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.nodes", "wt");
		printf("writing to file : out.nodes\n");
	}
	FileCloser fc2(fout);

	printf("printing nodes from file: %s\n\n",argv[1]);
	
	//find and print each node from fnodes file
	nodeReader nr;
	nr.setFile(fnodes);
	int nxtnode=nr.getNext(),printnum=0;

	while(0!=nxtnode){
		printnum++;
		if(doPrint)
			printf("printing node: %d\n",nxtnode);
		fprintf(fout,"%d",nxtnode);
		if(8==printnum){
			fprintf(fout,"\n");
			printnum=0;
		}
		else{
			fprintf(fout,", ");
		}
		nxtnode=nr.getNext();
	}

	printf("Finished printing nodes.\n");
	return 0;
}

