// sphereMapper.h: interface for the sphereMapper class.

#pragma once
#include <math.h>

class sphereMapper
{
private:
	struct POINT
	{
		double x,y,z;
	};
	double distance3D(double x1, double y1, double z1, double x2, double y2, double z2);
	bool coordToSphere(double* x, double* y, double* z);
	bool crossVectors(POINT first, POINT second, POINT* output);
	bool nearestNodes(POINT node, int* out1, int* out2, int* out3);

	int datanum;	//number of data points
	int nodeStart;	//number of first node in input files
	double* datax;	//array of x coords of data
	double* datay;	//array of y coords of data
	double* dataz;	//array of z coords of data
	double* pressure;  //array of pressure of data
	double* binvalues; //array of bin values for image
	double radius;	//radius of the sphere
	double a,b,c;	//coords of origin of the sphere
	double dx,dy;	//values of plane coordinate demensions
	
public:
	sphereMapper(int nodes,int nstart,int xres,int yres,double ox,double oy,double oz);
	virtual ~sphereMapper();

	bool loadNodesK(FILE* fk,FILE* fn);
	bool sphereFit(int maxi);
	bool toSphereCoords();
	bool toFemurCoords(double ax,double ay,double az,double lx,double ly,double lz,double px,double py,double pz);
	bool toPlaneCoords();
	bool loadPressures(FILE* fp,FILE* fn,int pressureStart);
	bool toImageBins(int xres,int yres);
	bool writeImage(FILE* fout,FILE* fe,int xres,int yres,int option);
	bool printSphereNodes(FILE* fout);
	bool printSphereElements(FILE* fout, FILE* fe);
	bool printCircle(FILE* fout,int xres,int yres);
	
};
