// sphereToCircle.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 12/14/04

#include "stdafx.h"

#pragma once
#include <iostream>
#include "sphereMapper.cpp"


// USAGE  prints usage message to screen
void usage()
{
    printf("Converts a 3d lsdyna .k file of a hemisphere to a 2D lsdyna .k file\n");
	printf("Usage: sphereToCircle [input mesh (.k)] [input pressures (.press)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -anterior <ax ay az (double)>\t\t= coordinates for anterior point of hemisphere circumference; default 'prompts user'\n");
	printf("  -efile <fe (.k)>\t\t\t= element connectivity to use for output; default 'uses 1st input file'\n");
	printf("  -i <maxitrs (int)>\t\t\t= number of itterations for fitting alogrithm; default '100'\n");
	printf("  -lateral <lx ly lz (double)>\t\t= coordinates for lateral point of hemisphere circumference; default 'prompts user'\n");
	printf("  -nfile <fnn,fpn (.nodes)>\t\t\t= list of nodes indexes to fit sphere to; default 'fits all nodes in 1st input file'\n");
	printf("  -nstart <nodeStart (int)>\t\t= number of first node in 1st input file; default '1'\n");
	printf("  -nnum <nodeNum (int)>\t\t\t= total number of nodes to fit; default 'prompts user'\n");
	printf("  -origin <ox oy oz (double)>\t\t= coordinates for initial origin of hemisphere; default '0.0 0.0 0.0'\n");
	printf("  -posterior <px py pz (double)>\t= coordinates for posterior of hemisphere circumference; default 'prompts user'\n");
	printf("  -print\t\t\t\t= prompts user for all print options; default 'prints 2D circle to output'\n");
	printf("  -pstart <pressureStart (int)>\t\t= number of first node in pressure file; default '1'\n");
	printf("  -o <fout (.k)>\t\t\t= output file name (.k); default 'out.k'\n");
	printf("  -res <xres yres (int)>\t\t= image resolution in pixels; default '200 200'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fk, *fp, *fe=NULL, *fnn=NULL, *fpn=NULL, *fout=NULL;
	int pressureStart=1,nodeStart=1,nodeNum=0,maxitrs=100;
	double ox=0.0,oy=0.0,oz=0.0,ax,ay,az,lx,ly,lz,px,py,pz;
	int xres=200,yres=200;

	fk = fail_or_open(argv[1], "rt");
	fp = fail_or_open(argv[2], "rt");
	FileCloser fc1(fk),fc2(fp);

	//get and save options from the user
	bool noEFile=true, noOutFile=true,noAnterior=true,noLateral=true,noPosterior=true,doPrintPrompt=false;
	char toConvert[20];
	for(int i=3;i<argc;i++){
		if(0==strcmp(argv[i],"-anterior")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&ax)){
				printf("entered invalid x for anterior");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&ay)){
				printf("entered invalid y for anterior");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+3]);
			if(1!=sscanf(toConvert,"%lf",&az)){
				printf("entered invalid z for anterior");
				usage();
				return -2;
			}
			else{
				noAnterior=false;
				i+=3;
			}
		}else if(0==strcmp(argv[i],"-efile")){
	        fe	= fail_or_open(argv[i+1], "rt");
			printf("elements in file : %s\n",argv[i+1]);
			noEFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-i")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&maxitrs)){
				printf("entered invalid number of itterations");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-lateral")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&lx)){
				printf("entered invalid x for lateral");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&ly)){
				printf("entered invalid y for lateral");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+3]);
			if(1!=sscanf(toConvert,"%lf",&lz)){
				printf("entered invalid z for lateral");
				usage();
				return -2;
			}
			else{
				noLateral=false;
				i+=3;
			}
		}else if(0==strcmp(argv[i],"-nfile")){
	        fnn	= fail_or_open(argv[i+1], "rt");
			fpn	= fail_or_open(argv[i+1], "rt");
			printf("useing only nodes in file : %s\n",argv[i+1]);
			i++;
		}else if(0==strcmp(argv[i],"-nstart")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeStart)){
				printf("entered invalid number for first node");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-nnum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeNum)){
				printf("entered invalid number for nodes");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-origin")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&ox)){
				printf("entered invalid x for origin");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&oy)){
				printf("entered invalid y for origin");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+3]);
			if(1!=sscanf(toConvert,"%lf",&oz)){
				printf("entered invalid z for origin");
				usage();
				return -2;
			}
			else
				i+=3;
		}else if(0==strcmp(argv[i],"-posterior")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&px)){
				printf("entered invalid x for posterior");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%lf",&py)){
				printf("entered invalid y for posterior");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+3]);
			if(1!=sscanf(toConvert,"%lf",&pz)){
				printf("entered invalid z for posterior");
				usage();
				return -2;
			}
			else{
				noPosterior=false;
				i+=3;
			}
		}else if(0==strcmp(argv[i],"-print")){
			doPrintPrompt=true;
		}else if(0==strcmp(argv[i],"-pstart")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&pressureStart)){
				printf("entered invalid number for first node in pressure file");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-o")){
	        fout	= fail_or_open(argv[i+1], "at");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-res")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&xres)){
				printf("entered invalid x for image resolution");
				usage();
				return -2;
			}
			strcpy(toConvert,argv[i+2]);
			if(1!=sscanf(toConvert,"%d",&yres)){
				printf("entered invalid y for image resolution");
				usage();
				return -2;
			}
			else
				i+=2;
		}
	}
	if(noEFile){
		fe	=  fail_or_open(argv[1], "rt");
		printf("elements in file : %s\n",argv[1]);
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "at");
		printf("writing to file : out.k\n");
	}
	if(noAnterior)
	{
		printf("\nPlease enter the coordinates of the anterior point. (x y z)\n");
		scanf("%lg%lg%lg",&ax,&ay,&az);
	}
	if(noLateral)
	{
		printf("\nPlease enter the coordinates of the lateral point. (x y z)\n");
		scanf("%lg%lg%lg",&lx,&ly,&lz);
	}
	if(noPosterior)
	{
		printf("\nPlease enter the coordinates of the posterior point. (x y z)\n");
		scanf("%lg%lg%lg",&px,&py,&pz);
	}
	if(nodeNum==0){
		printf("\nPlease enter the number of nodes to fit to sphere. (int)\n");
		scanf("%d",&nodeNum);
	}

	FileCloser fc3(fe),fc4(fnn),fc5(fpn),fc6(fout);
	printf("anterior coordinates: %lf %lf %lf\n",ax,ay,az);
	printf("max itterations: %d\n",maxitrs);
	printf("lateral coordinates: %lf %lf %lf\n",lx,ly,lz);
	printf("starting node # in input file: %d\n",nodeStart);
	printf("number of nodes to fit: %d\n",nodeNum);
	printf("origin coordinates: %lf %lf %lf\n",ox,oy,oz);
	printf("posterior coordinates: %lf %lf %lf\n",px,py,pz);
	printf("starting node # in pressure file: %d\n",pressureStart);
	printf("using circle resolution: %d x %d\n",xres,yres);
	printf("Reading from input file: %s\n",argv[1]);
	printf("Reading from pressure file: %s\n\n",argv[2]);

	// Query user for which output(s) to print
	int tempInt=-1;
	bool printLoad=false,printSphere=false,printFemur=false,printPlane=false,printElements=false,printCircle=false;
	if(doPrintPrompt){
		printf("Enter the number(s) for the outputs you would like to print.  When you are finished, enter 0.\n");
		printf("  1-nodes after Loading\n");
		printf("  2-nodes after Fitting to Sphere\n");
		printf("  3-nodes after Transposing to Femur Coordinates\n");
		printf("  4-nodes after Transposing to Plane Coordinates\n");
		printf("  5-solid element connectivities for Sphere input\n");
		printf("  6-nodes and shell element connectivities for Circle output\n");
		printf("  0-no output or finished\n");
		while(tempInt!=0){
			scanf("%d",&tempInt);
			switch (tempInt)
			{
			case 1:
				printLoad=true;
				break;
			case 2:
				printSphere=true;
				break;
			case 3:
				printFemur=true;
				break;
			case 4:
				printPlane=true;
				break;
			case 5:
				printElements=true;
				break;
			case 6:
				printCircle=true;
				break;
			}
		}
	}


	// Initialize a sphereMapper
	sphereMapper smap(nodeNum,nodeStart,xres,yres,ox,oy,oz);
	// Print header to output file
	fprintf(fout,"*KEYWORD\n");
	
	// Get coordinate data from fk (.k) file
	printf("\nLoading cordinate data ...\n");
	if(!smap.loadNodesK(fk,fnn)){
		printf("Failure loading nodes from .k file\n");
		return -3;
	}
	if(printLoad){
		printf("\nPrinting Nodes with current coordinates (appending)...\n");
		fprintf(fout,"$Load\n");
		if(!smap.printSphereNodes(fout)){
				printf("Failed to print sphere nodes\n");
				return -11;
		}
	}
	// Fit a sphere to the data
	printf("\nFitting Sphere to data ...\n");
	if(!smap.sphereFit(maxitrs)){
		printf("Faliled to find sphere before %d itterations\n",maxitrs);
		return -4;
	}
	// Transform data to Sphere Coordinates
	printf("\nTransforming data to Sphere coordinates ...\n");
	if(!smap.toSphereCoords()){
		printf("Failed to transform to sphere coordinates\n");
		return -5;
	}	
	if(printSphere){
		printf("\nPrinting Nodes with current coordinates (appending)...\n");
		fprintf(fout,"$Sphere\n");
		if(!smap.printSphereNodes(fout)){
				printf("Failed to print sphere nodes\n");
				return -11;
		}
	}
	// Transform data to Femur Coordinates
	printf("\nTransforming data to Femur coordinates ...\n");
	if(!smap.toFemurCoords(ax,ay,az,lx,ly,lz,px,py,pz)){
		printf("Failed to transform to femur coordinates\n");
		return -6;
	}
	if(printFemur){
		printf("\nPrinting Nodes with current coordinates (appending)...\n");
		fprintf(fout,"$Femur\n");
		if(!smap.printSphereNodes(fout)){
				printf("Failed to print sphere nodes\n");
				return -11;
		}
	}
	// Transform Data to Plane Coordinates
	printf("\nTransforming data to Plane coordinates ...\n");
	if(!smap.toPlaneCoords()){
		printf("Failed to transform to plane coordinates\n");
		return -7;
	}
	if(printPlane){
		printf("\nPrinting Nodes with current coordinates (appending)...\n");
		fprintf(fout,"$Plane\n");
		if(!smap.printSphereNodes(fout)){
				printf("Failed to print sphere nodes\n");
				return -11;
		}
	}
	if(printElements){
		printf("\nPrinting Elements of Sphere (appending)...\n");
		if(!smap.printSphereElements(fout,fe)){
			printf("Failed to print sphere elements\n");
			return -12;
		}
	}

	// Get pressure data from fp (pressure) file
	printf("\nLoading pressure data ...\n");
	if(!smap.loadPressures(fp,fpn,pressureStart)){
		printf("Failure loading pressures from file\n");
		return -8;
	}
	// Map Data into Image Bins
	printf("\nMapping data into Image Bins ...\n");
	if(!smap.toImageBins(xres, yres)){
		printf("Failed to map into image bins\n");
		return -9;
	}
	if(printCircle){
		printf("\nPrinting Nodes and Element Shells to 2D circle with current pressures (appending)...\n");
		if(!smap.printCircle(fout,xres,yres)){
			printf("Failed to print circle to file\n");
			return -13;
		}
	}	
	
	// Write Image
	if(!doPrintPrompt){
		if(!smap.writeImage(fout,fe,xres,yres,3)){
			printf("Failed to write 2D image to outfile.\n");
		return -14;
		}
	}

	fprintf(fout,"*END\n");
	return 0;
}

