// surfToOff.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 6/04/04

#include "stdafx.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <math.h>
#include <string.h>

// USAGE  prints usage message to screen
void usage()
{
	printf("Converts an amira surface file to an off file\n");
	printf("Usage: surfToOff [surface file (.surf)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -o <outfile (.off)> = output off file name; default 'out.off'\n");
	printf("  -print              = print debugging progress messages to screen\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

// MAIN converts file from .surf format to .off format
int main(int argc, char* argv[])
{
	printf("\n");
	if(NULL==argv[1]){
		usage();
		return -1;
	}

	FILE *fsurf, *foff, *fbuffer;
	bool doPrint=false;

	fsurf = fail_or_open(argv[1], "rt");
	FileCloser fc1(fsurf);

	//get and save options from the user
	bool noOutFile=true;
	for(int i=2;i<argc;i++){
		if(0==strcmp(argv[i],"-o")){
            foff	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
		}
	}
	if(noOutFile){
		foff	= fail_or_open("out.off", "wt");
		printf("writing to file : out.off\n");
	}
	FileCloser fc2(foff);

	printf("converting from file: %s\n\n",argv[1]);

	//initializing file
	char line[40];
	char temp[10];
	int vertices=0, triangles=0, edges=0;

	if(fsurf==0){
		printf("Failed to open file: %s\n",argv[1]);
		return -2;
	}
	
	// finding number of Vertices from .surf file
	if(NULL==fgets(line, sizeof(line), fsurf) ){
		printf("Failed to read first line of .surf file\n");
		return -3;
	}
	while (NULL==strstr(line,"Vertices")){
		if(NULL == fgets(line, sizeof(line), fsurf)){
			printf("Reached end of file before finding 'Vertices'\n");
			return -4;
		}
	}
	if(2!=sscanf(line,"%s %d", temp, &vertices)){
		printf("Couldn't read number of vertices\n");	
		return -5;
	}
	printf("vertices: %d\n",vertices);
	
	// saving values of vertices to buffer file
	fbuffer = fail_or_open("buffer.txt", "wb");
	for(int j=0;j<vertices;j++){
		fgets(line, sizeof(line), fsurf);
		fwrite(line, sizeof(char), strlen(line),fbuffer);
		if(doPrint)
            printf("scanning vertices line %d/%d  : %s",j+1,vertices,line);
	}
	fclose(fbuffer);

	// finding number of Triangles from .surf file
	while (NULL==strstr(line,"Triangles")){
		if(NULL == fgets(line, sizeof(line), fsurf)){
			printf("Reached end of file before finding 'Triangles'\n");
			return -6;
		}
	}
	if(2!=sscanf(line,"%s %d", temp, &triangles)){
		printf("Couldn't read number of triangles\n");	
		return -7;
	}
	printf("triangles: %d\n",triangles);

	// calculating number of Edges
	edges = triangles*3;
	printf("edges: %d\n",edges);

    // printing header to .off file
	fprintf(foff, "OFF\n");
	fprintf(foff, "%d %d %d\n", vertices, triangles, edges);
	if(doPrint){
		printf("printing header line 1/2    : OFF\n");
		printf("printing header line 2/2    : %d %d %d\n",vertices, triangles, edges);
	}

	// printing values of vertices from buffer file
	fbuffer = fail_or_open("buffer.txt", "rt");
	for(int k=0;k<vertices;k++){
		fgets(line,2,fbuffer);
		fgets(line, sizeof(line), fbuffer);
		if(doPrint)
			printf("printing vertices line %d/%d  : %s",k+1,vertices,line);
		fwrite(line, sizeof(char), strlen(line),foff);
	}
	fclose(fbuffer);

	// scanning and printing values of triangles from .surf to .off file
	int tempT[3];
	for(int m=0;m<triangles;m++){
		fgets(line, sizeof(line), fsurf);
		if(3!=sscanf(line, "%d %d %d", &tempT[0], &tempT[1], &tempT[2])){
			printf("Failed to read line");
			return -8; 
		}
		if(doPrint)
			printf("printing triangles line %d/%d : 3 %d %d %d\n",m+1,triangles,tempT[0]-1, tempT[1]-1, tempT[2]-1);
		fprintf(foff, "3 %d %d %d\n", tempT[0]-1, tempT[1]-1, tempT[2]-1);
	}
	
	printf("Finished printing to file.\n");
    return 0; 
}
