// thickElements.cpp : Defines the entry point for the console application.

// Written by Janna Balling
// 7/04/04

#include "stdafx.h"

#pragma once
#include <iostream>

#include "../nodeReader/nodeReader.cpp"

// USAGE  prints usage message to screen
void usage()
{
	printf("Creates new element layers around existing shells in an lsdyna .k file for given shell indexes (nodes) using given normal values\n"); 
    printf("Usage: thickElements [mesh input file (.k)] [mesh normals file (.vtk)] [truegrid shells file (.shells)] [OPTIONS]\n");
	printf("\nOPTIONS:\n");
	printf("  -copy                         = copy existing .k file before appending new nodes and elements\n"); 
	printf("  -num <totNum (int)>           = total number of new element layers to create, max '3'; default '1'\n");
	printf("  -node <nodeNum (int)>         = number of existing nodes; default 'promts user'\n");
	printf("  -nodeFile <nodeFile (.nodes)> = file name to print new surface nodes to; default 'out.nodes'\n"); 
	printf("  -enum <elementNum (int)>      = number to start numbering new elements; default '1'\n");
	printf("  -eval <elementVal (int)>      = material value to use for new elements; defalut '1'\n");
	printf("  -o <outfile (.k)>             = output file name; default 'out.k'\n");
	printf("  -print                        = print debugging progress messages to screen\n");
	printf("  -thick <totThick (double)>    = total thickness of all new element layers; defalut '1.0'\n");
	exit(-1);
}

// FAIL_OR_OPEN  opens file for reading(rt) or writing(wb)
FILE * fail_or_open(const char *path, const char *opt)
{
   FILE *f = fopen(path, opt);
   if(!f) {
     printf("Failure opening file %s\n", path);
     exit(-13);
   }
   return f;
}

// FILECLOSER  make sure open files get closed
struct FileCloser {   
   FileCloser(FILE *f) : m_file(f) {}
   ~FileCloser() { if(m_file) fclose(m_file); }
   FILE *m_file;
};

bool copyFile(FILE *fin, FILE *fout)
{
	if(fin==NULL||fout==NULL){
		printf("Falied to open files for copying\n");
		return false;
	}
	char line[1000];

	while(NULL!=fgets(line, sizeof(line), fin)){
		if(NULL!=strstr(line,"END")){
			break;
		}
		fputs(line,fout);
	}
	rewind(fin);
	return true;
}

struct oldNodes {
	double coords[3];
	bool isChanged;
	double norms[3];
};

struct newCords {
	double c[3];
};

bool storeNodes(FILE *fk,oldNodes *nd, int nnum)
{
	char line[1000];

	//find beginning of nodes in fk
	if(NULL==fgets(line, sizeof(line), fk) ){
		printf("Failed to read first line of fk (.k) file\n");
		return false;
	}
	while (NULL==strstr(line,"*NODE")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding nodes\n");
			return false;
		}
	}
	while(NULL!=strstr(line,"*")||NULL!=strstr(line,"$")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding nodes\n");
			return false;
		}
	}
	
	//store old node values for lookup
	for(int j=0;j<nnum;j++){
		if(3!=sscanf(line,"%*d%lf%lf%lf",&nd[j].coords[0],&nd[j].coords[1],&nd[j].coords[2])){
			printf("Failed to find nnum nodes in fk (.k) file\n");
			return false;
		}
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding nnum nodes\n");
			return false;
		}
	}

	return true;
}

bool storeNormals(FILE *fnorm,oldNodes *nd, int nnum,bool doPrint)
{
	char line[1000];

	//find beginning of normals in fnorm
	if(NULL==fgets(line, sizeof(line), fnorm) ){
		printf("Failed to read first line of fnorm (.vtk) file\n");
		return false;
	}
	while (NULL==strstr(line,"NORMALS")){
		if(NULL == fgets(line, sizeof(line), fnorm)){
			printf("Reached end of fnorm (.vtk) file before finding normals\n");
			return false;
		}
	}
	
	//store normal values for lookup
	for(int j=0;j<nnum;j++){
		if(3!=fscanf(fnorm,"%lf%lf%lf",&nd[j].norms[0],&nd[j].norms[1],&nd[j].norms[2])){
			printf("Failed to find nnum normals in fnorm (.vtk) file\n");
			return false;
		}
		if(doPrint&&j<100){
			printf("normal #%d: %f %f %f\n",j+1,nd[j].norms[0],nd[j].norms[1],nd[j].norms[2]);
		}
	}

	return true;
}

bool computeElments(FILE *fk,FILE *fnodes,FILE *fout
					,oldNodes *nd,newCords *nd1,newCords *nd2,newCords *nd3
					,int eNum,int eVal,int nNum,double tThick,int tNum)
{
	char line[1000];

	//find beginning of shells in fk
	while (NULL==strstr(line,"*ELEMENT_SHELL_THICKNESS")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding shells\n");
			return false;
		}
	}
	while(NULL!=strstr(line,"*")||NULL!=strstr(line,"$")){
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding shells\n");
			return false;
		}
	}

	//get each shell to use for new element
	nodeReader nr;
	nr.setFile(fnodes);
	int nxtelement=nr.getNext();
	int eVals[5]={0};
	int currentE=eNum;
	double sThick=tThick/tNum;
	double addX=0,addY=0,addZ=0;
	int layerNum=0;
	int nodeIndex=0;

	fprintf(fout,"*ELEMENT_SOLID\n");
	while(0!=nxtelement){
		if(5!=sscanf(line,"%d%*d%d%d%d%d",&eVals[0],&eVals[1],&eVals[2],&eVals[3],&eVals[4])){
			printf("Failed to find all shells from fnodes (.nodes) file in fk (.k) file\n");
			return false;
		}
		//print new elment
		if(nxtelement==eVals[0]){
			for(int k=1;k<5;k++){
				nodeIndex=eVals[k]-1;
				//calculate values for new cords
				if(!nd[nodeIndex].isChanged){
					addX=nd[nodeIndex].norms[0];
						if(addX<0.0) addX=(addX*addX);
						else addX = -(addX*addX);
					addY=nd[nodeIndex].norms[1];
						if(addY<0.0) addY=(addY*addY);
						else addY = -(addY*addY);
					addZ=nd[nodeIndex].norms[2];
						if(addZ<0.0) addZ=(addZ*addZ);
						else addZ = -(addZ*addZ);
					layerNum=1;
					if(layerNum<=tNum){
						nd1[nodeIndex].c[0]=nd[nodeIndex].coords[0]+(addX*layerNum*sThick);
						nd1[nodeIndex].c[1]=nd[nodeIndex].coords[1]+(addY*layerNum*sThick);
						nd1[nodeIndex].c[2]=nd[nodeIndex].coords[2]+(addZ*layerNum*sThick);
					}
					layerNum=2;
					if(layerNum<=tNum){
						nd2[nodeIndex].c[0]=nd[nodeIndex].coords[0]+(addX*layerNum*sThick);
						nd2[nodeIndex].c[1]=nd[nodeIndex].coords[1]+(addY*layerNum*sThick);
						nd2[nodeIndex].c[2]=nd[nodeIndex].coords[2]+(addZ*layerNum*sThick);
					}
					layerNum=3;
					if(layerNum<=tNum){
						nd3[nodeIndex].c[0]=nd[nodeIndex].coords[0]+(addX*layerNum*sThick);
						nd3[nodeIndex].c[1]=nd[nodeIndex].coords[1]+(addY*layerNum*sThick);
						nd3[nodeIndex].c[2]=nd[nodeIndex].coords[2]+(addZ*layerNum*sThick);
					}
					nd[nodeIndex].isChanged=true;
				}
			}
			//print new elements
			for(layerNum=1;layerNum<=tNum;layerNum++){
				fprintf(fout,"%8d%8d%8d%8d%8d%8d%8d%8d%8d%8d\n"
					,currentE++,eVal
					,eVals[3]+(nNum*(layerNum-1)),eVals[2]+(nNum*(layerNum-1)),eVals[1]+(nNum*(layerNum-1)),eVals[1]+(nNum*(layerNum-1))
					,eVals[3]+(nNum*layerNum),eVals[2]+(nNum*layerNum),eVals[1]+(nNum*layerNum),eVals[1]+(nNum*layerNum)
					);	
			}
			nxtelement=nr.getNext();
		}
		//skip thickness values
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding all shells from fnodes (.nodes) file\n");
			return false;
		}
		//get next element line
		if(NULL == fgets(line, sizeof(line), fk)){
			printf("Reached end of fk (.k) file before finding all shells from fnodes (.nodes) file\n");
			return false;
		}
	}

	return true;
}

bool printNodes(FILE *fout,oldNodes *nd,newCords *nd1,newCords *nd2,newCords *nd3,int nNum,int tNum,const char* nFile)
{
	FILE* fnodesOut = fopen(nFile,"wt");
	if(fnodesOut==NULL){
		printf("Falied to open nodeFile for printing\n");
		return false;
	}
	
	int layerNum=0;
	int numChanged=0;

	fprintf(fout,"*NODE\n");
	for(int m=0;m<nNum;m++){
		if(nd[m].isChanged){
			numChanged++;
			layerNum=1;
			if(layerNum<=tNum){
				fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",m+1+(nNum*layerNum),nd1[m].c[0],nd1[m].c[1],nd1[m].c[2]);
			}
			layerNum=2;
			if(layerNum<=tNum){
				fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",m+1+(nNum*layerNum),nd2[m].c[0],nd2[m].c[1],nd2[m].c[2]);
			}
			layerNum=3;
			if(layerNum<=tNum){
				fprintf(fout,"%8d%16.8lf%16.8lf%16.8lf\n",m+1+(nNum*layerNum),nd3[m].c[0],nd3[m].c[1],nd3[m].c[2]);
			}
			fprintf(fnodesOut,"%d",m+1+(nNum*tNum));
			if((numChanged)%8==0)
				fprintf(fnodesOut,"\n");
			else
				fprintf(fnodesOut,", ");
		}
	}
	
	fprintf(fnodesOut,"\n$ Made new elements for %d existing nodes\n",numChanged);
	return true;
}

int main(int argc, char* argv[])
{
	printf("\n");
	if(argc<3) {
		usage();
		return -1;
	}

	FILE *fk,*fnorm,*fnodes,*fout;
	bool doCopy=false,doPrint=false;
	int totNum=1,nodeNum=0;
	char *nodeFile = "out.nodes";
	int elementNum=1,elementVal=1;
	double totThick=1.0;

	fk		= fail_or_open(argv[1], "rt");
	fnorm	= fail_or_open(argv[2], "rt");
	fnodes	= fail_or_open(argv[3], "rt");
	FileCloser fc1(fk),fc2(fnorm),fc3(fnodes);
	
	//get and save options from the user
	char toConvert[20];
	bool noOutFile=true;
	for(int i=4;i<argc;i++){
		if(0==strcmp(argv[i],"-copy")){
			doCopy=true;
		}else if(0==strcmp(argv[i],"-num")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&totNum)){
				printf("entered invalid number for tot number of element layers\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-node")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&nodeNum)){
				printf("entered invalid number of nodes\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-nodeFile")){
			nodeFile=argv[i+1];
			i++;
		}else if(0==strcmp(argv[i],"-enum")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&elementNum)){
				printf("entered invalid number to start elment index\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-eval")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%d",&elementVal)){
				printf("entered invalid number for element material value\n");
				usage();
				return -2;
			}
			else
				i++;
		}else if(0==strcmp(argv[i],"-o")){
            fout	= fail_or_open(argv[i+1], "wt");
			printf("writing to file : %s\n",argv[i+1]);
			noOutFile=false;
			i++;
		}else if(0==strcmp(argv[i],"-print")){
			doPrint=true;
			printf("printing debug messages\n");
		}else if(0==strcmp(argv[i],"-thick")){
			strcpy(toConvert,argv[i+1]);
			if(1!=sscanf(toConvert,"%lf",&totThick)){
				printf("entered invalid number for total thickness of new element layers\n");
				usage();
				return -2;
			}
			else
				i++;
		}
	}
	if(noOutFile){
		fout	= fail_or_open("out.k", "wt");
		printf("writing to file : out.k\n");
	}
	if(nodeNum==0){
		printf("\nPlease enter the number of nodes in input file. (int)\n");
		scanf("%d",&nodeNum);
	}

	FileCloser fc4(fout);
	printf("creating %d layers\n",totNum);
	printf("layers have a total thickness of %f\n",totThick);
	printf("adding %d to existing node number\n",nodeNum); 
	printf("printing new surface nodes to file %s\n",nodeFile);
	printf("starting element numbering at %d\n",elementNum);
	printf("assigning element value of %d\n\n",elementVal);
	
	//copy original file before appending new elements.
	if(doCopy){
        printf("copying existing .k file ...\n");
		if(!copyFile(fk,fout) ){
			printf("Failure copying file\n");
			return -3;
		}
	}

	//make space to store node info
    oldNodes *nd=new oldNodes[nodeNum];
	for(int j=0;j<nodeNum;j++){
		nd[j].isChanged=false;
	}
	newCords *nd1=NULL,*nd2=NULL,*nd3=NULL;
	if(totNum>=3) nd3=new newCords[nodeNum];
	if(totNum>=2) nd2=new newCords[nodeNum];
	if(totNum>=1) nd1=new newCords[nodeNum];

	//store old node values for lookup
	printf("getting nodes from .nodes file ...\n");
	if(!storeNodes(fk,nd,nodeNum)){
		printf("Failure storing nodes\n");
		return -4;
	}

	//store normals for lookup
	printf("getting normals from .vtk file ...\n");
	if(!storeNormals(fnorm,nd,nodeNum,doPrint)){
		printf("Failure storing normals\n");
		return -5;
	}

	//look up shells and make new elements
	printf("computing new elements for shells from .k file ...\n");
	if(!computeElments(fk,fnodes,fout,nd,nd1,nd2,nd3,elementNum,elementVal,nodeNum,totThick,totNum)){
		printf("Failure computing elments\n");
		return -6;
	}

	//make new nodes for those we changed
	printf("printing new nodes used by new elements ...\n");
	if(!printNodes(fout,nd,nd1,nd2,nd3,nodeNum,totNum,nodeFile)){
		printf("Failure printing nodes\n");
		return -7;
	}

	fprintf(fout,"*END\n");

	delete [] nd;
	delete [] nd1;
	delete [] nd2;
	delete [] nd3;
	printf("Done ...\n");
	return 0;
}

